<?php
// $Id: bubbletimer.statistics.inc,v 1.2 2009/02/24 09:34:00 itarato Exp $


/**
 * @file
 * Bubbletimer statistic reports
 * Features:
 * 	- csv export
 */


/**
 * Page rendering
 * @return string
 */
function bubbletimer_statistics_page() {
  drupal_add_js('misc/collapse.js');
  $output = t('
  	<p>Create your report. Select the interval and one user to put into the query.</p>
  	<p><strong>Default interval is the current month.</strong></p>
  ');
  $output .= drupal_get_form('bubbletimer_statistics_form');
  
  return $output;
}


/**
 * Form definition
 * @param array $form_state
 * @return string
 */
function bubbletimer_statistics_form($form_state) {
  global $user;
  
  $form['bt_fieldset_default'] = array(
    '#type' => 'fieldset',
    '#title' => 'Options',
  );
  $form['bt_fieldset_default']['bt_stat_date_from'] = array(
    '#title' => t('From'),
    '#type' => 'date',
    '#default_value' => array(
      'day' => 1,
      'month' => date('n'),
      'year' => date('Y'),
    ),
  );
  $form['bt_fieldset_default']['bt_stat_date_to'] = array(
    '#title' => t('To'),
    '#type' => 'date',
    '#default_value' => array(
      'day' => date('t', time()),
      'month' => date('n'),
      'year' => date('Y'),
    ),
  );
  
  if (user_access(TT_PERM_STAT_ANY, $user)) {
    $result = db_query('SELECT uid, name FROM {users} WHERE length(name) > 0 ORDER BY uid ASC;');
    $users = array();
    while ($row = db_fetch_object($result)) {
      $users[$row->uid] = $row->name;
    }
    $form['bt_fieldset_default']['bt_user'] = array(
      '#type' => 'select',
      '#title' => t('Select user(s)'),
      '#options' => $users,
      '#default_value' => $user->uid,
    );
  }
  
  $form['bt_fieldset_extra'] = array(
    '#type' => 'fieldset',
    '#title' => 'Format options',
    '#attributes' => array('class' => 'collapsible collapsed'),
  );
  $decimal_separators = _bt_get_decimal_separators();
  $form['bt_fieldset_extra']['bt_decimal_separator'] = array(
    '#type' => 'select',
    '#title' => 'Decimal separator',
    '#options' => $decimal_separators,
  );
  
  $form['bt_stat_submit'] = array(
    '#type' => 'submit',
    '#value' => 'Generate',
  );
  
  return $form;
}


/**
 * Implementation of hook_submit()
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_statistics_form_submit($form, $form_state) {
  global $user;
    
  $date_from = 
    $form_state['values']['bt_stat_date_from']['year'] .'-'.
    $form_state['values']['bt_stat_date_from']['month'] .'-'.
    $form_state['values']['bt_stat_date_from']['day'];
    
  $date_to = 
    $form_state['values']['bt_stat_date_to']['year'] .'-'.
    $form_state['values']['bt_stat_date_to']['month'] .'-'.
    $form_state['values']['bt_stat_date_to']['day'];

  if (user_access(TT_PERM_STAT_ANY, $user) && !empty($form_state['values']['bt_user'])) {
    $uid = intval($form_state['values']['bt_user']);
  }
  else {
    $uid = $user->uid; 
  }
  
  $decimal_separators= _bt_get_decimal_separators();
  $decimal_separator = substr($decimal_separators[$form_state['values']['bt_decimal_separator']], 0, 1);
  
  bubbletimer_create_csv($date_from, $date_to, $uid, $decimal_separator);
}


/**
 * Generate CSV file from submitted parameters and terminate as a downloadable file
 * @param string $date_from
 * @param string $date_to
 * @param integer $uid
 */
function bubbletimer_create_csv($date_from, $date_to, $uid, $decimal_separator = '.') {
  $resource = db_query(
    'SELECT t.*, u.name, n.title
  	FROM ({bubbletimer_activity} t LEFT JOIN {users} u ON t.uid = u.uid) LEFT JOIN {node} n ON n.nid = t.nid
  	WHERE t.date >= \'%s\' AND t.date <= \'%s\' AND t.uid = %d 
  	ORDER BY t.date ASC;',
    $date_from,
    $date_to,
    $uid
  );
  
  $result = array();
  $nodes = array();
  while ($row = db_fetch_object($resource)) {
    $result[strtotime($row->date)][$row->nid] = bt_activity_hour_total($row->activity);
    if (!array_key_exists($row->nid, $nodes)) {
      $nodes[$row->nid] = $row->title;
    }
  }

  $user = user_load(array('uid' => $uid));
  $user_name = $user->name;

  if (empty($result)) {
    drupal_set_message(t('No information found about @name from @from to @to.',array('@name' => $user_name, '@from' => $date_from, '@to' => $date_to)));
    return;
  }

  $csv = '"'. $user_name .'\'s activity",';
  $start_ts = strtotime($date_from);
  $end_ts = strtotime($date_to);
  foreach ((array)$nodes as $title) {
    $csv .= '"'. $title .'",';
  }
  $csv .= "\n";
  for ($timestamp = $start_ts; $timestamp <= $end_ts; $timestamp += BT_DAY) {
    $csv .= date('Y-m-d,', $timestamp);
    foreach ((array)$nodes as $nid => $title) {
      if (empty($result[$timestamp][$nid])) {
        $value = 0;
      }
      else {
        $value = number_format($result[$timestamp][$nid], 2, $decimal_separator, '');
      }
      $csv .= '"'. $value .'",';
    }
    $csv .= "\n";
  }
  
  // Create downloadable content
  drupal_set_header('Content-Type: text/x-comma-separated-values');
  drupal_set_header('Content-Disposition: attachment; filename="bubbletimer_'. $user_name .'_'. $date_from .'-'. $date_to   .'.csv"');
  echo $csv;
  module_invoke_all('exit');
  exit;
}


/**
 * Count time from activity pattern
 * 0 means nothing
 * 1 means 15 minutes
 * @param string $pattern
 * @return float
 */
function bt_activity_hour_total($pattern) {
  $length = strlen($pattern);
  $total = 0;
  for ($i = 0; $i < $length; $i++) {
    if (substr($pattern, $i, 1) == '1') {
      $total += 0.25;
    }
  }
  return $total;
}


/**
 * Decimal separator array
 * Drupal can't get global arrays
 * First char of the values are the separators
 * @return array
 */
function _bt_get_decimal_separators() {
  return array(
    1 => '. (period)',
    2 => ', (comma)',
  );
}
